@@ -23,6 +23,8 @@
 
 #include <gtk/gtk.h>
 
+#include "egg-action-group.h"
+#include "egg-menu-merge.h"
 #include "kz-mozembed.h"
 #include "kz-window.h"
 
