@@ -28,6 +28,7 @@
 #include "egg-menu-merge.h"
 #include "kz-mozembed.h"
 #include "kz-gesture.h"
+#include "kz-embed-event.h"
 
 G_BEGIN_DECLS
 
@@ -93,6 +94,7 @@ struct _KzWindow
 	/* other objects */
 	GtkAccelGroup  *accel_group;
 	EggActionGroup *action_group;
+	EggActionGroup *popup_action_group;
 	EggMenuMerge   *menu_merge;
 	KzGestureItems *gesture_items;
 
@@ -141,6 +143,9 @@ void              kz_window_update_nav_buttons       (KzWindow *kz,
 						      KzMozEmbed *current);
 GNode            *kz_window_get_tree                 (KzWindow *kz);
 
+const KzEmbedEventMouse *
+		  kz_window_get_mouse_event_info     (KzWindow *kz);
+
 G_END_DECLS
 
 #endif /* __KZ_WINDOW_H__ */
