@@ -24,6 +24,7 @@
 
 #include <gtk/gtk.h>
 #include <gtkmozembed.h>
+#include "kz-embed-event.h"
 
 G_BEGIN_DECLS
 
@@ -66,72 +67,13 @@ struct _KzMozEmbedClass
 };
 
 
-/**
- * MozillaKeyModifier: key modifiers
- */
-typedef enum
-{
-	ALT_KEY   = (1 << 0),
-	CTRL_KEY  = (1 << 1),
-	SHIFT_KEY = (1 << 2),
-	META_KEY  = (1 << 3),
-	KEY_CODE  = (1 << 4)
-} MozillaKeyModifier;
-
-#define CONTEXT_NONE     0
-#define CONTEXT_DEFAULT  1
-#define CONTEXT_LINK     2
-#define CONTEXT_IMAGE    4
-#define CONTEXT_DOCUMENT 8
-#define CONTEXT_INPUT    64
-#define CONTEXT_OTHER    128
-#define CONTEXT_XUL      256
-
-
-typedef enum
-{
-	MOZILLA_EVENT_KEY,
-	MOZILLA_EVENT_MOUSE
-} KzMozEventType;
-
-typedef struct
-{
-	KzMozEventType type;
-
-	gchar *link;
-	gchar *linktext;
-	gchar *img;
-
-	/*
-	gboolean framed_page;
-	gint context;
-	*/
-} KzMozEvent;
-
-typedef struct
-{
-	KzMozEvent cinfo;
-	gint key;
-	gint modifier;
-	gint char_code;
-} KzMozEventKey;
-
-typedef struct
-{
-	KzMozEvent cinfo;
-	gint button;
-	gint modifier;
-	gint x, y;
-
-	/* guint timestamp; */
-} KzMozEventMouse;
-
-
 GtkType      kz_moz_embed_get_type         (void);
 GtkWidget   *kz_moz_embed_new              (const gchar *url);
 
 void         kz_moz_embed_load_url         (KzMozEmbed  *kzembed,
 					    const gchar *url);
+void         kz_moz_embed_view_source      (KzMozEmbed  *kzembed);
+
 #define      kz_moz_embed_is_loading(embed) \
                  (KZ_IS_MOZ_EMBED(embed) ? KZ_MOZ_EMBED(embed)->is_loading : FALSE)
 
@@ -157,17 +99,12 @@ gboolean     kz_moz_embed_get_links          (KzMozEmbed  *kzembed,
 					      GList      **list,
 					      gboolean     selected_only);
 
-#if 1 /* FIXME!!!!! */
 glong     kz_moz_embed_get_key_event_info   (KzMozEmbed     *kzembed,
 					     gpointer        event,
-					     KzMozEventKey **info);
+					     KzEmbedEventKey **info);
 glong     kz_moz_embed_get_mouse_event_info (KzMozEmbed     *kzembed,
 					     gpointer        event,
-					     KzMozEventMouse **info);
-KzMozEvent *
-          kz_moz_embed_event_new            (KzMozEventType type);
-void      kz_moz_embed_event_free           (KzMozEvent    *cinfo);
-#endif
+					     KzEmbedEventMouse **info);
 
 G_END_DECLS
 
