@@ -20,6 +20,7 @@
  */
 
 #include "gtk-utils.h"
+#include <string.h>
 
 /**
  * menu_position_under_widget:
@@ -199,3 +200,15 @@ gtkutil_list_store_swap(GtkListStore *store, GtkTreeIter  *a, GtkTreeIter  *b)
 	g_free(order);
 #endif /* (GTK_MAJOR_VERSON >= 2) && (GTK_MINOR_VERSION >= 2) */
 }
+
+void
+gtkutil_copy_text (const gchar *text)
+{
+	if (text && *text)
+	{
+		GtkClipboard *clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+		gtk_clipboard_set_text(clipboard, text, strlen(text));
+		clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+		gtk_clipboard_set_text(clipboard, text, strlen(text));
+	}
+}
